﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

using Trolleybus.DrawingObjects;
using Trolleybus.MovementStrategy;

namespace Trolleybus.Generics
{
    internal class BusesGenericStorage
    {
        /// <summary>
        /// Словарь (хранилище)
        /// </summary>
        readonly Dictionary<string, BusesGenericCollection<DrawingBus, DrawingObjectBus>> _busStorages;

        /// <summary>
        /// Возвращение списка названий наборов
        /// </summary>
        public List<string> Keys => _busStorages.Keys.ToList();

        /// <summary>
        /// Ширина окна отрисовки
        /// </summary>
        private readonly int _pictureWidth;

        /// <summary>
        /// Высота окна отрисовки
        /// </summary>
        private readonly int _pictureHeight;

        /// <summary>
        /// Конструктор
        /// </summary>
        /// <param name="pictureWidth"></param>
        /// <param name="pictureHeight"></param>
        public BusesGenericStorage(int pictureWidth, int pictureHeight)
        {
            _busStorages = new Dictionary<string, BusesGenericCollection<DrawingBus, DrawingObjectBus>>();
            _pictureWidth = pictureWidth;
            _pictureHeight = pictureHeight;
        }

        /// <summary>
        /// Добавление набора
        /// </summary>
        /// <param name="name">Название набора</param>
        public void AddSet(string name)
        {
            // проверка, существует ли набор с таким ключём
            if (_busStorages.ContainsKey(name))
            {
                return;
            }
            var busCollection = new BusesGenericCollection<DrawingBus, DrawingObjectBus>(_pictureWidth, _pictureHeight);
            _busStorages.Add(name, busCollection);
        }

        /// <summary>
        /// Удаление набора
        /// </summary>
        /// <param name="name">Название набора</param>
        public void DelSet(string name)
        {
            if (_busStorages.ContainsKey(name))
            {
                _busStorages.Remove(name);
            }
        }
        /// <summary>
        /// Доступ к набору
        /// </summary>
        /// <param name="ind"></param>
        /// <returns></returns>
        public BusesGenericCollection<DrawingBus, DrawingObjectBus>? this[string ind]
        {
            get
            {
                if (!_busStorages.ContainsKey(ind))
                {
                    return null;
                }
                return _busStorages[ind];
            }
        }
    }
}
