﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

using Trolleybus.DrawingObjects;
using Trolleybus.Generics;
using Trolleybus.MovementStrategy;

namespace Trolleybus
{
    /// <summary>
    /// Форма для работы с набором объектов класса DrawingBus
    /// </summary>
    public partial class FormBusesCollection : Form
    {
        /// <summary>
        /// Набор объектов
        /// </summary>
        private readonly BusesGenericStorage _storage;
        /// <summary>
        /// Конструктор
        /// </summary>
        public FormBusesCollection()
        {
            InitializeComponent();
            _storage = new BusesGenericStorage(pictureBoxCollection.Width, pictureBoxCollection.Height);
        }

        /// <summary>
        /// Заполнение listBoxObjects
        /// </summary>
        private void ReloadObjects()
        {
            int index = listBoxSets.SelectedIndex;
            listBoxSets.Items.Clear();
            for (int i = 0; i < _storage.Keys.Count; i++)
            {
                listBoxSets.Items.Add(_storage.Keys[i]);
            }
            if (listBoxSets.Items.Count > 0 && (index == -1 || index >= listBoxSets.Items.Count))
            {
                listBoxSets.SelectedIndex = 0;
            }
            else if (listBoxSets.Items.Count > 0 && index > -1 && index < listBoxSets.Items.Count)
            {
                listBoxSets.SelectedIndex = index;
            }
        }
        /// <summary>
        /// Добавление набора в коллекцию
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void ButtonAddSetOfObjects_Click(object sender, EventArgs e)
        {
            if (string.IsNullOrEmpty(textBoxNameOfSet.Text))
            {
                MessageBox.Show("Не все данные заполнены", "Ошибка",
                MessageBoxButtons.OK, MessageBoxIcon.Error);
                return;
            }
            _storage.AddSet(textBoxNameOfSet.Text);
            ReloadObjects();
        }
        /// <summary>
        /// Выбор набора
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void ListBoxSets_SelectedIndexChanged(object sender, EventArgs e)
        {
            pictureBoxCollection.Image = _storage[listBoxSets.SelectedItem?.ToString() ?? string.Empty]?.ShowBuses();
        }
        /// <summary>
        /// Удаление набора
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void ButtonDeleteSetOfObjects_Click(object sender, EventArgs e)
        {
            if (listBoxSets.SelectedIndex == -1)
            {
                return;
            }
            if (MessageBox.Show($"Удалить объект {listBoxSets.SelectedItem}?", "Удаление",
                MessageBoxButtons.YesNo, MessageBoxIcon.Question) == DialogResult.Yes)
            {
                _storage.DelSet(listBoxSets.SelectedItem.ToString() ?? string.Empty);
                ReloadObjects();
            }
        }
        /// <summary>
        /// Добавление объекта в набор
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void ButtonAddBus_Click(object sender, EventArgs e)
        {
            if (listBoxSets.SelectedIndex == -1)
            {
                return;
            }
            var obj = _storage[listBoxSets.SelectedItem.ToString() ?? string.Empty];
            if (obj == null)
            {
                return;
            }

            FormTrolleybus form = new FormTrolleybus();
            if (form.ShowDialog() == DialogResult.OK)
            {
                if (obj + form.SelectedBus != -1)
                {
                    MessageBox.Show("Объект добавлен");
                    pictureBoxCollection.Image = obj.ShowBuses();
                }
                else
                {
                    MessageBox.Show("Не удалось добавить объект");
                }
            }
        }
        /// <summary>
        /// Удаление объекта из набора
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void ButtonRemoveBus_Click(object sender, EventArgs e)
        {
            if (listBoxSets.SelectedIndex == -1)
            {
                return;
            }
            var obj = _storage[listBoxSets.SelectedItem.ToString() ?? string.Empty];
            if (obj == null)
            {
                return;
            }
            if (MessageBox.Show("Удалить объект?", "Удаление",
            MessageBoxButtons.YesNo, MessageBoxIcon.Question) == DialogResult.No)
            {
                return;
            }
            String pos_string = maskedTextBoxNumber.Text;
            if (pos_string == "")
            {
                pos_string = "0";
            }
            int pos = Convert.ToInt32(pos_string);
            if (obj - pos)
            {
                MessageBox.Show("Объект удален");
                pictureBoxCollection.Image = obj.ShowBuses();
            }
            else
            {
                MessageBox.Show("Не удалось удалить объект");
            }
        }
        /// <summary>
        /// Обновление рисунка по набору
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void ButtonRefreshCollection_Click(object sender, EventArgs e)
        {
            if (listBoxSets.SelectedIndex == -1)
            {
                return;
            }
            var obj = _storage[listBoxSets.SelectedItem.ToString() ?? string.Empty];
            if (obj == null)
            {
                return;
            }
            pictureBoxCollection.Image = obj.ShowBuses();
        }

        private void InitializeComponent()
        {
            pictureBoxCollection = new PictureBox();
            panelTools = new Panel();
            panelSets = new Panel();
            buttonDeleteSetOfObjects = new Button();
            textBoxNameOfSet = new TextBox();
            listBoxSets = new ListBox();
            buttonAddSetOfObjects = new Button();
            buttonRefreshCollection = new Button();
            maskedTextBoxNumber = new MaskedTextBox();
            buttonRemoveBus = new Button();
            buttonAddBus = new Button();
            ((System.ComponentModel.ISupportInitialize)pictureBoxCollection).BeginInit();
            panelTools.SuspendLayout();
            panelSets.SuspendLayout();
            SuspendLayout();
            // 
            // pictureBoxCollection
            // 
            pictureBoxCollection.Anchor = AnchorStyles.Top | AnchorStyles.Bottom | AnchorStyles.Left | AnchorStyles.Right;
            pictureBoxCollection.Location = new Point(0, 0);
            pictureBoxCollection.Name = "pictureBoxCollection";
            pictureBoxCollection.Size = new Size(670, 553);
            pictureBoxCollection.TabIndex = 0;
            pictureBoxCollection.TabStop = false;
            // 
            // panelTools
            // 
            panelTools.Anchor = AnchorStyles.Top | AnchorStyles.Bottom | AnchorStyles.Right;
            panelTools.BorderStyle = BorderStyle.FixedSingle;
            panelTools.Controls.Add(panelSets);
            panelTools.Controls.Add(buttonRefreshCollection);
            panelTools.Controls.Add(maskedTextBoxNumber);
            panelTools.Controls.Add(buttonRemoveBus);
            panelTools.Controls.Add(buttonAddBus);
            panelTools.Location = new Point(682, 0);
            panelTools.Name = "panelTools";
            panelTools.Size = new Size(200, 553);
            panelTools.TabIndex = 1;
            // 
            // panelSets
            // 
            panelSets.Anchor = AnchorStyles.Top | AnchorStyles.Right;
            panelSets.BorderStyle = BorderStyle.FixedSingle;
            panelSets.Controls.Add(buttonDeleteSetOfObjects);
            panelSets.Controls.Add(textBoxNameOfSet);
            panelSets.Controls.Add(listBoxSets);
            panelSets.Controls.Add(buttonAddSetOfObjects);
            panelSets.Location = new Point(17, 11);
            panelSets.Name = "panelSets";
            panelSets.Size = new Size(170, 283);
            panelSets.TabIndex = 4;
            // 
            // buttonDeleteSetOfObjects
            // 
            buttonDeleteSetOfObjects.Location = new Point(3, 222);
            buttonDeleteSetOfObjects.Name = "buttonDeleteSetOfObjects";
            buttonDeleteSetOfObjects.Size = new Size(162, 45);
            buttonDeleteSetOfObjects.TabIndex = 3;
            buttonDeleteSetOfObjects.Text = "Удалить набор";
            buttonDeleteSetOfObjects.UseVisualStyleBackColor = true;
            buttonDeleteSetOfObjects.Click += ButtonDeleteSetOfObjects_Click;
            // 
            // textBoxNameOfSet
            // 
            textBoxNameOfSet.Location = new Point(3, 19);
            textBoxNameOfSet.Name = "textBoxNameOfSet";
            textBoxNameOfSet.Size = new Size(162, 27);
            textBoxNameOfSet.TabIndex = 2;
            // 
            // listBoxSets
            // 
            listBoxSets.FormattingEnabled = true;
            listBoxSets.ItemHeight = 20;
            listBoxSets.Location = new Point(3, 112);
            listBoxSets.Name = "listBoxSets";
            listBoxSets.Size = new Size(162, 104);
            listBoxSets.TabIndex = 1;
            listBoxSets.SelectedIndexChanged += ListBoxSets_SelectedIndexChanged;
            // 
            // buttonAddSetOfObjects
            // 
            buttonAddSetOfObjects.Location = new Point(3, 52);
            buttonAddSetOfObjects.Name = "buttonAddSetOfObjects";
            buttonAddSetOfObjects.Size = new Size(162, 45);
            buttonAddSetOfObjects.TabIndex = 0;
            buttonAddSetOfObjects.Text = "Добавить набор";
            buttonAddSetOfObjects.UseVisualStyleBackColor = true;
            buttonAddSetOfObjects.Click += ButtonAddSetOfObjects_Click;
            // 
            // buttonRefreshCollection
            // 
            buttonRefreshCollection.Location = new Point(17, 484);
            buttonRefreshCollection.Name = "buttonRefreshCollection";
            buttonRefreshCollection.Size = new Size(170, 40);
            buttonRefreshCollection.TabIndex = 3;
            buttonRefreshCollection.Text = "Обновить коллекцию";
            buttonRefreshCollection.UseVisualStyleBackColor = true;
            buttonRefreshCollection.Click += ButtonRefreshCollection_Click;
            // 
            // maskedTextBoxNumber
            // 
            maskedTextBoxNumber.Location = new Point(39, 366);
            maskedTextBoxNumber.Mask = "00";
            maskedTextBoxNumber.Name = "maskedTextBoxNumber";
            maskedTextBoxNumber.Size = new Size(125, 27);
            maskedTextBoxNumber.TabIndex = 2;
            maskedTextBoxNumber.ValidatingType = typeof(int);
            // 
            // buttonRemoveBus
            // 
            buttonRemoveBus.Location = new Point(17, 409);
            buttonRemoveBus.Name = "buttonRemoveBus";
            buttonRemoveBus.Size = new Size(170, 40);
            buttonRemoveBus.TabIndex = 1;
            buttonRemoveBus.Text = "Удалить автобус";
            buttonRemoveBus.UseVisualStyleBackColor = true;
            buttonRemoveBus.Click += ButtonRemoveBus_Click;
            // 
            // buttonAddBus
            // 
            buttonAddBus.Location = new Point(17, 311);
            buttonAddBus.Name = "buttonAddBus";
            buttonAddBus.Size = new Size(170, 40);
            buttonAddBus.TabIndex = 0;
            buttonAddBus.Text = "Добавить автобус";
            buttonAddBus.UseVisualStyleBackColor = true;
            buttonAddBus.Click += ButtonAddBus_Click;
            // 
            // FormBusesCollection
            // 
            ClientSize = new Size(882, 553);
            Controls.Add(panelTools);
            Controls.Add(pictureBoxCollection);
            Name = "FormBusesCollection";
            Text = "Набор автобусов";
            ((System.ComponentModel.ISupportInitialize)pictureBoxCollection).EndInit();
            panelTools.ResumeLayout(false);
            panelTools.PerformLayout();
            panelSets.ResumeLayout(false);
            panelSets.PerformLayout();
            ResumeLayout(false);
        }

        private PictureBox pictureBoxCollection;
        private Panel panelTools;
        private Button buttonRefreshCollection;
        private MaskedTextBox maskedTextBoxNumber;
        private Button buttonRemoveBus;
        private Panel panelSets;
        private Button buttonAddSetOfObjects;
        private ListBox listBoxSets;
        private TextBox textBoxNameOfSet;
        private Button buttonDeleteSetOfObjects;
        private Button buttonAddBus;

    }
}
